$! Extract table and domain information from an Rdb Database.
$! Write the information in RTF format for input to a word processor.
$!
$! B. Z. Lederman	06-Nov-1991
$!			12-Nov-1991	Add domains, look at comments
$!			18-Nov-1991	Add cross reference
$!			20-Nov-1991	no cross reference for views
$!
$ IF F$MODE () .EQS. "BATCH" THEN SET PROCESS /NAME = "Rdb to RTF"
$!
$ ON ERROR        THEN $ GOTO exitout
$ ON CONTROL_Y    THEN $ GOTO exitout
$ SET NOON
$!
$ IF F$TYPE (SQL$) .EQS. "" THEN $ SQL$ :== $SYS$SYSTEM:SQL$
$!
$ FALSE  = 0
$ TRUE   = -1
$ num_tab = 0
$ wanted      = FALSE
$ source_mode = FALSE
$ table_name = "Dummy Table"
$!
$ IF P1 .EQS. ""			! if no input file specified
$ THEN
$    P1 = F$TRNLNM ("SQL$DATABASE")	! look for a default
$    IF P1 .EQS. ""			! no default
$    THEN
$	WRITE SYS$OUTPUT "Can't do anything without a database"
$	EXIT
$    ENDIF
$ ENDIF
$!
$ IF P2 .EQS. ""			! if no output file specified
$ THEN
$    report_file = "rdb_database.rtf"
$ ELSE
$    report_file = P2
$ ENDIF
$!
$! Parse the filespec to remove the type and replace it with ".RDB"
$!
$ file_name = F$PARSE (P1,,,"NAME")
$ file_dir  = F$PARSE (P1,,,"DIRECTORY")
$ rdb_file = file_dir + file_name + ".RDB"
$!
$ this_file = F$SEARCH (rdb_file, 1)
$ IF this_file .NES. ""  THEN $ GOTO got_file
$ WRITE SYS$OUTPUT "The file ''rdb_file doesn't exist"
$ EXIT
$!
$ got_file:
$!
$ DEFINE /NOLOG SQL$DATABASE 'this_file'
$!
$ IF F$SEARCH ("rdb_tables.temp1") .NES. "" THEN -
  DELETE /NOLOG rdb_tables.temp1;*
$ IF F$SEARCH ("rdb_tables.temp2") .NES. "" THEN -
  DELETE /NOLOG rdb_tables.temp2;*
$ IF F$SEARCH ("rdb_tables.temp3") .NES. "" THEN -
  DELETE /NOLOG rdb_tables.temp3;*
$ IF F$SEARCH ("rdb_tables.temp4") .NES. "" THEN -
  DELETE /NOLOG rdb_tables.temp4;*
$ IF F$SEARCH ("rdb_cross.temp3") .NES. "" THEN -
  DELETE /NOLOG rdb_cross.temp3;*
$ IF F$SEARCH ("rdb_cross.temp4") .NES. "" THEN -
  DELETE /NOLOG rdb_cross.temp4;*
$ IF F$SEARCH ("rdb_fetch_tables_temp.sql") .NES. "" THEN -
  DELETE /NOLOG rdb_fetch_tables_temp.sql;*
$ IF F$SEARCH ("rdb_fetch_domains_temp.sql") .NES. "" THEN -
  DELETE /NOLOG rdb_fetch_domains_temp.sql;*
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 1"
$ ELSE
$    WRITE SYS$OUTPUT "About to get list of tables and domains"
$ ENDIF
$!
$! First get the tables.
$!
$ DEFINE SYS$OUTPUT NL:
$!
$ SQL$
SET OUTPUT rdb_tables.temp1
SHOW TABLES
SET OUTPUT rdb_tables.temp3
SHOW DOMAINS
EXIT
$!
$! Check exit status to see if there was a database access error?
$!
$ DEASSIGN SYS$OUTPUT
$!
$ OPEN /WRITE /ERROR = nofile outfile 'report_file'
$ WRITE outfile "{\rtf1\mac\deff2 {\fonttbl{\f0\fswiss Chicago;}{\f2\froman New York;}"
$ WRITE outfile "{\f3\fswiss Geneva;}{\f4\fmodern Monaco;}{\f16\fnil Palatino;}"
$ WRITE outfile "{\f20\froman Times;}{\f21\fswiss Helvetica;}{\f22\fmodern Courier;}"
$ WRITE outfile "{\f23\ftech Symbol;}}{\colortbl\red0\green0\blue0;\red0\green0\blue255;"
$ WRITE outfile "\red0\green255\blue255;\red0\green255\blue0;\red255\green0\blue255;"
$ WRITE outfile "\red255\green0\blue0;\red255\green255\blue0;\red255\green255\blue255;}"
$ WRITE outfile "{\stylesheet{\sbasedon222\snext0 Normal;}}"
$ WRITE outfile "\margl1440\margr1440\widowctrl\ftnbj \sectd \linemod0\linex0\cols1\endnhere"
$ WRITE outfile "\pard\plain\fs20"
$ WRITE outfile "{\b Database " + this_file + "\par"
$ WRITE OUTFILE F$FAO ("!11<!%D!>\par\par}{\b\ul Domains}\par\par", 0)
$ WRITE outfile "\pard \tx2800\tx4600"
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 2"
$ ELSE
$    WRITE SYS$OUTPUT "Parsing domain names."
$ ENDIF
$!
$ OPEN /READ /ERROR = nofile tmpfile rdb_tables.temp3
$!
$ OPEN /WRITE /ERROR = nofile cmdfile rdb_fetch_domains_temp.sql
$ cmdopen = TRUE
$ WRITE cmdfile "SET OUTPUT rdb_tables.temp4"
$!
$! Get domains
$!
$ readloop2:
$!
$ READ /END = endfile2 tmpfile instring
$!
$ tmpstring = F$EDIT (instring, "COMPRESS, TRIM, UPCASE")
$!
$! If the line is too short (there are some blank lines) get the next line
$!
$ IF F$LENGTH (tmpstring) .LE. 1 THEN GOTO readloop2
$!
$! Parse out parts of the line.
$!
$ domain = F$ELEMENT (0, " ", tmpstring)
$!
$ IF domain .EQS. "SET"				! finish on SET OUTPUT
$ THEN
$    second = F$ELEMENT (1, " ", tmpstring)
$    IF second .EQS. "OUTPUT" THEN GOTO endfile1
$ ENDIF
$!
$ IF domain .EQS. "USER"			! reject the first line
$ THEN
$    second = F$ELEMENT (1, " ", tmpstring)
$    third  = F$ELEMENT (2, " ", tmpstring)
$    IF second .EQS. "DOMAINS" .AND. third .EQS. "IN" THEN GOTO readloop2
$ ENDIF
$!
$ IF domain .EQS. "SHOW"			! reject the SHOW command
$ THEN
$    IF F$ELEMENT (1, " ", tmpstring) .EQS. "DOMAINS" THEN GOTO readloop2
$ ENDIF
$!
$ IF domain .EQS. "EXIT" THEN GOTO endfile2
$!
$ WRITE cmdfile "SHOW DOMAIN " + domain
$!
$ GOTO readloop2
$!
$ endfile2:
$!
$ CLOSE tmpfile
$!
$ WRITE cmdfile "EXIT"
$ CLOSE cmdfile
$ cmdopen = FALSE
$ PURGE /NOLOG rdb_fetch_domains_temp.sql
$!
$ DEFINE SYS$OUTPUT NL:
$!
$ SQL$ @rdb_fetch_domains_temp.sql
$!
$ DEASSIGN SYS$OUTPUT
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 3"
$ ELSE
$    WRITE SYS$OUTPUT "About to append domains."
$ ENDIF
$!
$ OPEN /READ /ERROR = nofile tmpfile rdb_tables.temp4
$!
$ readloop3:
$!
$ READ /END = endfile3 tmpfile instring
$!
$ tmpstring = F$EDIT (instring, "COMPRESS, TRIM, UPCASE")
$!
$ first  = F$ELEMENT (0, " ", tmpstring)
$ second = F$ELEMENT (1, " ", tmpstring)
$!
$ IF first .EQS. "SHOW" .AND. second .EQS. "DOMAIN" THEN $ GOTO readloop3
$!
$ IF first .EQS. "EXIT" THEN $ GOTO endfile3
$!
$! The line with the field name must be formatted differently than
$! the comment and missing value lines. None of the field name lines
$! I've seen have a colon (":") in them, so that's what I'll use
$! to tell one line from another.
$!
$ IF F$LOCATE (":", tmpstring) .EQ. F$LENGTH (tmpstring)	! no colon
$ THEN							! must be field
$    field = F$ELEMENT (0, " ", tmpstring)
$    rest = tmpstring - field
$    WRITE outfile "\par"
$    WRITE outfile "{\b " + field + "}\tab" + rest + "\par"
$    GOTO readloop3
$ ENDIF
$!
$! Should probably parse here for tabs and replace with spaces or "\tab",
$! though it looks like tab characters are acceptable in RTF.  May have
$! to account for multi-line comments, however.
$!
$ WRITE outfile instring + "\par"
$!
$ GOTO readloop3
$!
$ endfile3:
$!
$ CLOSE tmpfile
$!
$! Get Tables
$!
$ OPEN /READ /ERROR = nofile tmpfile rdb_tables.temp1
$!
$ OPEN /WRITE /ERROR = nofile cmdfile rdb_fetch_tables_temp.sql
$ cmdopen = TRUE
$ WRITE cmdfile "SET OUTPUT rdb_tables.temp2"
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 4"
$ ELSE
$    WRITE SYS$OUTPUT "Parsing table names"
$ ENDIF
$!
$ WRITE outfile "\pard \page \par"
$ WRITE outfile "{\b Database " + this_file + "\par"
$ WRITE OUTFILE F$FAO ("!11<!%D!>\par\par}{\b\ul Tables}\par\par", 0)
$ WRITE outfile "\trowd \trgaph80\trleft-80 \cellx4600\cellx9280\pard \intbl\tx2500 "
$!
$ readloop1:
$!
$ READ /END = endfile1 tmpfile instring
$!
$ tmpstring = F$EDIT (instring, "COMPRESS, TRIM")
$!
$! If the line is too short (there are some blank lines) get the next line
$!
$ IF F$LENGTH (tmpstring) .LE. 1 THEN GOTO readloop1
$!
$! Parse out parts of the line.
$!
$ table = F$ELEMENT (0, " ", tmpstring)
$!
$ IF table .EQS. "SET"				! finish on SET OUTPUT
$ THEN
$    second = F$ELEMENT (1, " ", tmpstring)
$    IF second .EQS. "OUTPUT" THEN GOTO endfile1
$ ENDIF
$!
$ IF table .EQS. "User"				! reject the first line
$ THEN
$    second = F$ELEMENT (1, " ", tmpstring)
$    third  = F$ELEMENT (2, " ", tmpstring)
$    IF second .EQS. "tables" .AND. third .EQS. "in" THEN GOTO readloop1
$ ENDIF
$!
$ IF table .EQS. "SHOW"				! reject the SHOW command
$ THEN
$    IF F$ELEMENT (1, " ", tmpstring) .EQS. "TABLES" THEN GOTO readloop1
$ ENDIF
$!
$ IF table .EQS. "EXIT" THEN GOTO endfile1
$!
$ WRITE cmdfile "SHOW TABLE " + table
$!
$! Convert "xxx_VIEW     A view." to "xxx_VIEW\tab A view.\par"
$!
$ IF F$LOCATE ("A view", tmpstring) .NE. F$LENGTH (tmpstring)	! found view
$ THEN
$    tmpstring = F$ELEMENT (0, " ", tmpstring) + "\tab A View."
$ ENDIF
$!
$ WRITE outfile tmpstring + "\par"
$!
$! Count number of tables and insert column break when needed
$!
$ num_tab = num_tab + 1
$ IF num_tab .EQ. 45 THEN $ WRITE outfile "\cell \pard \intbl\tx2500"
$!
$ GOTO readloop1
$!
$ endfile1:
$!
$ CLOSE tmpfile
$!
$ WRITE cmdfile "EXIT"
$ CLOSE cmdfile
$ cmdopen = FALSE
$ PURGE /NOLOG rdb_fetch_tables_temp.sql
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 5"
$ ELSE
$    WRITE SYS$OUTPUT "About to get individual tables"
$ ENDIF
$!
$ DEFINE SYS$OUTPUT NL:
$!
$ SQL$ @rdb_fetch_tables_temp.sql
$!
$ DEASSIGN SYS$OUTPUT
$!
$ PURGE /NOLOG rdb_tables.temp*
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 6"
$ ELSE
$    WRITE SYS$OUTPUT "About to process each table"
$ ENDIF
$!
$ OPEN /READ /ERROR = nofile tmpfile rdb_tables.temp2
$!
$! If we didn't write anything into the right-hand column we must
$! do so now, or else the remainder of the document goes into
$! the right-hand colum and MS Word hangs.
$!
$ IF num_tab .LT. 45 THEN $ WRITE outfile "\cell \pard \intbl\tx2500 \par { } \par"
$ WRITE outfile "\cell \pard \row \par"
$!
$ OPEN /WRITE /ERROR = nofile crossfile rdb_cross.temp3
$ crossfile_open = TRUE
$!
$ readloop4:
$!
$ READ /END = endfile4 tmpfile instring
$!
$ IF F$LENGTH (instring) .LE. 2 THEN GOTO readloop4	! skip if too short
$!
$ IF instring .EQS. "EXIT" THEN $ GOTO endfile4
$!
$ select = F$EXTRACT (0, 10, instring)
$!
$ IF select .EQS. "Table cons"
$ THEN
$    wanted      = FALSE
$    source_mode = FALSE
$    GOTO readloop4
$ ENDIF
$!
$ IF select .EQS. "SHOW TABLE"
$ THEN
$    table_name = F$ELEMENT (2, " ", instring)
$    wanted      = FALSE
$    source_mode = FALSE
$    WRITE SYS$OUTPUT table_name		! so we can see progress
$    GOTO readloop4
$ ENDIF
$!
$ IF select .EQS. "----------"
$ THEN
$    wanted = TRUE
$    WRITE outfile "\page\par\pard \tx2800\tx4600"
$    WRITE outfile "{\b Columns for " + table_name + "}\par\par"
$    WRITE outfile "{\ul Column Name}{\ul \tab }{\ul Data Type}{\ul \tab}{\ul Domain}\par"
$    GOTO readloop4
$ ENDIF
$!
$ IF .NOT. wanted THEN $ GOTO readloop4		! not ready for output yet
$!
$! Have to extract field names from everything else.  Fields start
$! in column 1 and have a blank space (or should have) in columns 31
$! and 32.
$!
$ col1  = F$EXTRACT ( 0, 1, instring)
$ col31 = F$EXTRACT (30, 1, instring)
$!
$ IF col1 .NES. " " .AND. col31 .EQS. " "
$ THEN
$    tmpstring = F$EDIT (F$ELEMENT (0, "	", instring), "TRIM")
$    domain = F$EDIT (F$ELEMENT (1, "	", instring), "COLLAPSE, TRIM")
$    IF domain .EQS. "" THEN domain = F$EDIT (F$ELEMENT (2, "	", instring), "COLLAPSE, TRIM")
$    column = F$ELEMENT (0, " ", tmpstring)
$    length = F$LENGTH (tmpstring) - 32
$    data_t = F$EXTRACT (32, length, tmpstring)
$    IF domain .NES. ""				! no cross reference for views
$    THEN
$	WRITE crossfile domain + "," + table_name + "." + column + "," + data_t
$    ENDIF
$    WRITE outfile "\par"
$    WRITE outfile "{\b " + column + "}\tab " + data_t + "\tab " + domain
$    GOTO readloop4
$ ENDIF
$!
$! Still have to look for the start of a source line.
$!
$ select = F$EXTRACT (0, 8, select)
$ IF select .EQS. " Source:"
$ THEN
$    WRITE outfile "\par\par { Source:}"
$!
$! Strip the <CR><LF> out of the line
$!
$    WRITE outfile "\par" + F$EXTRACT (10, F$LENGTH(instring) - 10, instring)
$    source_mode = TRUE
$    GOTO readloop4
$ ENDIF
$!
$! Have to look for Missing Values, which are not part of comments.
$! Start them on their own line. Also look for View Source lines.
$!
$ IF select .EQS. " Missing" .OR. select .EQS. " Comment" .OR. source_mode
$ THEN
$    WRITE outfile "\par" + instring
$    GOTO readloop4
$ ENDIF
$!
$! Again, might parse tabs out of comments and similar fields, though
$! RTF appears to accept tab characters.
$!
$ WRITE outfile instring
$ GOTO readloop4
$!
$ endfile4:
$!
$ CLOSE tmpfile
$ CLOSE crossfile 
$ crossfile_open = FALSE
$ SORT /STAT rdb_cross.temp3 rdb_cross.temp4
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 7"
$ ELSE
$    WRITE SYS$OUTPUT "About process sorted cross-reference"
$ ENDIF
$!
$ OPEN /READ /ERROR = nofile crossfile rdb_cross.temp4
$ crossfile_open = TRUE
$ OPEN /WRITE /ERROR = nofile outfile 'report_file'
$ WRITE outfile "\pard \tx2800\tx6300"
$ WRITE outfile "\page\par{\b Cross reference Domains and Tables}\par\par"
$ WRITE outfile "{\b \ul Domain\tab Table.Column\tab Data Type}\par\par"
$!
$ readloop5:
$!
$ READ /END = endfile5 crossfile instring
$!
$ domain     = F$ELEMENT (0, ",", instring)
$ table_name = F$ELEMENT (1, ",", instring)
$ data_t     = F$ELEMENT (2, ",", instring)
$!
$ WRITE outfile "\par " + domain + "\tab " + table_name + "\tab " + data_t
$!
$ GOTO readloop5
$!
$ endfile5:
$!
$!
$ exitout:
$!
$ SET NOON
$!
$ IF cmdopen THEN $ CLOSE cmdfile
$ IF crossfile_open THEN $ CLOSE crossfile
$ IF cmdopen THEN $ CLOSE cmdfile
$!
$ WRITE outfile "\par}}"
$ CLOSE outfile
$!
$ DELETE /NOLOG rdb_tables.temp1;*
$ DELETE /NOLOG rdb_tables.temp2;*
$ DELETE /NOLOG rdb_tables.temp3;*
$ DELETE /NOLOG rdb_tables.temp4;*
$ DELETE /NOLOG rdb_cross.temp3;*
$ DELETE /NOLOG rdb_cross.temp4;*
$ DELETE /NOLOG rdb_fetch_tables_temp.sql;*
$ DELETE /NOLOG rdb_fetch_domains_temp.sql;*
$!
$ nofile:
$!
$ EXIT $STATUS
