$! Parse out XFC Cache data
$!
$! B. Z. Lederman  	08-May-2003
$! 			21-Nov-2003	add selection of device
$!
$ IF F$MODE() .EQS. "BATCH" THEN SET PROCESS/NAME="XFC Parsing"
$!
$ ON CONTROL_Y THEN $ GOTO endfile
$!
$ IF P1 .NES. ""
$ THEN
$    volume = P1
$ ELSE
$    volume = "SYS$LOGIN_DEVICE"
$ ENDIF
$!
$! The following should be an easy way to insure there is
$! one and only one colon at the end of the device name
$!
$ devname = F$GETDVI (volume, "DISPLAY_DEVNAM")
$!
$ filename = devname - ":"
$!
$ SHOW MEMORY /CACHE = FILE = 'devname'*.* /OUT = 'filename'.log
$ EDI /TPU /NOSECT /NOINI /NODISP /COMMAND = xfc_cache_log_fix.tpu 'filename'.log
$ SORT /STAT /NODUP 'filename'.log 'filename'.log ! optional
$!
$ OPEN /WRITE /ERROR = nofile outfile 'filename'.CSV
$ OPEN /READ  /ERROR = nofile inpfile 'filename'.LOG
$!
$ ON ERROR        THEN $ GOTO endfile
$ ON CONTROL_Y    THEN $ GOTO endfile
$!
$ header = "Node,Device,Directory,File,Open,Enabled,Active Caching Mode,Allocated Pages,Total QIOs,Read Hits,Virtual Reads,Virtual Writes,Hit Rate %,Read Aheads,Read Throughs,Write Throughs,Read Arounds,Write Arounds"
$ WRITE outfile header
$!
$ readloop:
$!
$ READ /END = endfile inpfile instring
$!
$! Parse out first part of the line.
$!
$ firstc = F$EXTRACT (0, 1, instring)
$!
$ IF firstc .EQS. "_"			! device names can have leading "_"
$ THEN
$!
$! Use F$PARSE to separate the full file specification into separate
$! fields
$!
$    fullspec = F$ELEMENT (0, ",", instring)
$    device = F$PARSE (fullspec,,, "DEVICE")
$    node = F$ELEMENT (0, "$", device) - "_"
$    nonode = F$ELEMENT (1, "$", device)
$    directory = F$PARSE (fullspec,,, "DIRECTORY")
$!
$! There isn't a "file specification" field type, so the name, type
$! and version have to be pulled out individually, then re-combined.
$!
$    name = F$PARSE (fullspec,,, "NAME")
$    type = F$PARSE (fullspec,,, "TYPE")
$    version = F$PARSE (fullspec,,, "VERSION")
$    file = name + type + version
$!
$! Everything else remains the same
$!
$    tmpstring = instring - fullspec
$    revised = node + "," + nonode + "," + directory + "," + file
$!
$! Put the processed string back together
$!
$    outstring = revised + tmpstring
$    WRITE outfile outstring
$    GOTO readloop
$ ENDIF
$!
$!
$! Skip everything else
$!
$ GOTO readloop
$!
$ endfile:
$!
$ CLOSE outfile
$ CLOSE inpfile
$!
$ nofile:
$!
$ EXIT $STATUS
